//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#import "CategoriesViewController.h"
#import "ContentViewController.h"
#import "ContentDetailsViewController.h"
#import "CategoryCell.h"
#import "CategoryCellActionDelegate.h"
#import "FetchServicesResult.h"
#import "FetchCategoryServicesResult.h"
#import "Resources.h"

static NSString *const CellIdentifier = @"CategoryCell";

@interface CategoriesViewController () <UITableViewDataSource, UITableViewDelegate, UISearchBarDelegate, CategoryCellActionDelegate>

@property (nonatomic, strong) NSArray<YCSStreamingService *> *services;
@property (nonatomic, strong) NSArray<YCSStreamingCategory *> *categories;
@property (nonatomic, strong) NSMutableArray<FetchCategoryServicesResult *> *categoriesData;
@property (nonatomic, strong) UISearchBar *searchBar;
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) UIActivityIndicatorView *activityIndicator;
@property (nonatomic, strong) UIActivityIndicatorView *cancelActivityIndicator;
@property (nonatomic, strong) UIButton *cancelButton;
@property (nonatomic, strong) UIView* cancelView;

@property (nonatomic, strong) YCSConnect *ycsConnect;
@property (nonatomic, strong) YCSCancellationTicket *fetchContentCancellationTicket;

@end

@implementation CategoriesViewController

- (nonnull instancetype)initWithServices:(nonnull NSArray<YCSStreamingService *> *)services;
{
    if (self = [super init])
        self.services = services;

    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.ycsConnect = [YCSConnect shared];
    [self setupUI];
}

- (void)viewWillAppear:(BOOL)animated
{
    [self refresh:^
    {
    }];
}

- (void)refresh:(void(^)(void))completionHandler
{
    [self.activityIndicator startAnimating];
    [self loadCategories:^
    {
        [self.tableView reloadData];
        [self.activityIndicator stopAnimating];
        self.cancelView.hidden = FALSE;

        [self loadServices:^
        {
            self.cancelView.hidden = TRUE;
            [self.cancelActivityIndicator stopAnimating];

            //All done
            if (completionHandler)
                completionHandler();

            for (int i = 0; i <= self.categoriesData.count; i++)
            {
                NSIndexPath *path = [NSIndexPath indexPathForRow:i inSection:0];
                CategoryCell *cell = (CategoryCell *)[self.tableView cellForRowAtIndexPath:path];
                [cell stopLoading];
            }
        }];
    }];
}

- (void)loadCategories:(void(^)(void))completionHandler
{
    [self.ycsConnect fetchCategories:nil completionHandler:^(NSArray<YCSStreamingCategory *> *categories, NSError *error)
    {
        self.categoriesData = [[NSMutableArray<FetchCategoryServicesResult *> alloc] init];
        for (YCSStreamingCategory *category in categories)
        {
            FetchCategoryServicesResult *categoryData = [[FetchCategoryServicesResult alloc] init];
            categoryData.category = category;
            categoryData.services = [[NSMutableArray<FetchServicesResult *> alloc] init];
            [self.categoriesData addObject:categoryData];
        }

        self.categories = categories;
        if (completionHandler)
            completionHandler();
    }];
}

- (void)loadServices:(void(^)(void))completionHandler
{
    self.fetchContentCancellationTicket = [[YCSCancellationTicket alloc] init];

    [self.ycsConnect fetchContent:self.categories services:self.services cancellationTicket:self.fetchContentCancellationTicket
    serviceErrorHandler:^(YCSStreamingService *service, NSError *error)
    {
        //Don't use alert here. It's for demonstration purposes only
        UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Warning" message:[error localizedDescription] preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *okButton = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil];
        [alert addAction:okButton];
        [self presentViewController:alert animated:YES completion:nil];
    }
    categoryServiceCompletionHandler:^(YCSStreamingCategory *category, YCSStreamingService *service, NSArray<YCSStreamingContent *> *content, NSError *error)
    {
        FetchCategoryServicesResult *categoryData = [self getCategoryData:category];
        if (categoryData == nil)
            return;

        FetchServicesResult *serviceData = nil;
        NSPredicate *servicePredicate = [NSPredicate predicateWithFormat:@"service.name==%@", service.name];
        NSArray<FetchServicesResult *> *serviceResults = [categoryData.services filteredArrayUsingPredicate:servicePredicate];
        if (serviceResults.count > 0)
            serviceData = serviceResults[0];

        if (serviceData == nil)
        {
            serviceData = [[FetchServicesResult alloc] init];
            serviceData.service = service;
            serviceData.content = [[NSMutableArray<YCSStreamingContent *> alloc] init];
            [categoryData.services addObject:serviceData];
        }

        [serviceData.content addObjectsFromArray:content];

        NSInteger categoryIndex = [self.categoriesData indexOfObject:categoryData];
        NSIndexPath *path = [NSIndexPath indexPathForRow:categoryIndex inSection:0];
        CategoryCell *cell = (CategoryCell *)[self.tableView cellForRowAtIndexPath:path];
        [cell reloadData];
    }
    allCompletionHandler:^(NSArray<FetchCategoryServicesResult *> *result, NSError *error)
    {
        if (completionHandler)
            completionHandler();

        if (error != nil)
            [self showError:error];
    }];
}

- (void)showError:(NSError *)error
{
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Error" message:[error localizedDescription] preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *okButton = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil];
    [alert addAction:okButton];
    [self presentViewController:alert animated:YES completion:nil];
}

- (FetchCategoryServicesResult *)getCategoryData:(YCSStreamingCategory *)category
{
    NSPredicate *categoryPredicate = [NSPredicate predicateWithFormat:@"category.name==%@", category.name];
    NSArray<FetchCategoryServicesResult *> *categoryResults = [self.categoriesData filteredArrayUsingPredicate:categoryPredicate];
    if (categoryResults.count > 0)
        return categoryResults[0];

    return nil;
}

- (void)setupUI
{
    self.cancelView = [[UIView alloc] init];
    self.cancelView.backgroundColor = [UIColor blackColor];
    self.cancelView.hidden = TRUE;
    [self.view addSubview:self.cancelView];
    self.cancelView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.cancelView.topAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.topAnchor constant:0],
        [self.cancelView.leftAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.leftAnchor constant:8],
        [self.cancelView.rightAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.rightAnchor constant:-8],
        [self.cancelView.heightAnchor constraintEqualToConstant:30]]];

    self.cancelButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.cancelButton setTitleColor:[UIColor colorWithRed:32.0/255.0 green:32.0/255.0 blue:32.0/255.0 alpha:1.0] forState:UIControlStateNormal];
    self.cancelButton.backgroundColor = [Resources mainColor];
    self.cancelButton.layer.borderColor = [UIColor clearColor].CGColor;
    self.cancelButton.layer.borderWidth = 0.0f;
    self.cancelButton.layer.cornerRadius = 10.0f;
    self.cancelButton.titleLabel.font = [UIFont systemFontOfSize:12];
    [self.cancelButton addTarget:self action:@selector(cancelClicked:) forControlEvents:UIControlEventTouchUpInside];
    [self.cancelButton setTitle:@"Cancel" forState:UIControlStateNormal];
    [self.cancelView addSubview:self.cancelButton];
    self.cancelButton.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.cancelButton.widthAnchor constraintEqualToConstant:120],
        [self.cancelButton.heightAnchor constraintEqualToConstant:30],
        [self.cancelButton.topAnchor constraintEqualToAnchor:self.cancelView.topAnchor constant:0],
        [self.cancelButton.rightAnchor constraintEqualToAnchor:self.cancelView.rightAnchor constant:0]]];

    self.cancelActivityIndicator = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleMedium];
    self.cancelActivityIndicator.color = [UIColor whiteColor];
    [self.cancelView addSubview:self.cancelActivityIndicator];
    self.cancelActivityIndicator.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.cancelActivityIndicator.topAnchor constraintEqualToAnchor:self.cancelButton.topAnchor constant:5],
        [self.cancelActivityIndicator.leftAnchor constraintEqualToAnchor:self.cancelButton.leftAnchor constant:5]]];

    self.searchBar = [[UISearchBar alloc] init];
    self.searchBar.searchBarStyle = UISearchBarStyleMinimal;
    self.searchBar.barStyle = UIBarStyleBlack;
    self.searchBar.tintColor = [UIColor whiteColor];
    self.searchBar.delegate = self;
    [self.view addSubview:self.searchBar];
    self.searchBar.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.searchBar.topAnchor constraintEqualToAnchor:self.cancelView.bottomAnchor constant:0],
        [self.searchBar.leftAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.leftAnchor constant:0],
        [self.searchBar.rightAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.rightAnchor constant:0],
        [self.searchBar.heightAnchor constraintEqualToConstant:60]]];

    self.tableView = [[UITableView alloc] initWithFrame:CGRectZero style:UITableViewStylePlain];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.backgroundColor = [UIColor clearColor];
    self.tableView.rowHeight = 140;

    [self.tableView registerClass:CategoryCell.self forCellReuseIdentifier:CellIdentifier];
    [self.view addSubview:self.tableView];
    self.tableView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.tableView.topAnchor constraintEqualToAnchor:self.searchBar.bottomAnchor constant:0],
        [self.tableView.leftAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.leftAnchor constant:self.searchBar.directionalLayoutMargins.leading],
        [self.tableView.rightAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.rightAnchor constant:0],
        [self.tableView.bottomAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.bottomAnchor constant:0]]];

    self.view.backgroundColor = [UIColor blackColor];
}

- (void)cancelClicked:(UIButton *)sender
{
    if (self.fetchContentCancellationTicket != nil)
        [self.fetchContentCancellationTicket cancel];

    [self.cancelActivityIndicator startAnimating];
}

#pragma mark - Table view data source

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.categoriesData.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    CategoryCell *cell = (CategoryCell *)[tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil)
        cell = [[CategoryCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];

    FetchCategoryServicesResult *categoryData = self.categoriesData[indexPath.row];
    [cell updateWithCategoryData:categoryData];
    cell.actionDelegate = self;

    return cell;
}

#pragma mark - Category Cell Action Delegate

- (void)categoryCellActionBrowse:(id)sender service:(YCSStreamingService *)service content:(YCSStreamingContent *)content
{
    ContentViewController *browseView = [[ContentViewController alloc] init];
    [self presentViewController:browseView animated:TRUE completion:^
    {
        [browseView browse:service content:content completionHandler:^
        {
        }];
    }];
}

- (void)categoryCellActionDetails:(id)sender service:(YCSStreamingService *)service content:(YCSStreamingContent *)content
{
    ContentDetailsViewController *detailsView = [[ContentDetailsViewController alloc] initWithContent:content];
    [self presentViewController:detailsView animated:TRUE completion:nil];
}

#pragma mark - SearchBar Delegate

-(void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText
{
}

- (void)searchBarTextDidBeginEditing:(UISearchBar *)searchBar
{
    searchBar.showsCancelButton = YES;
}

- (void)searchBarTextDidEndEditing:(UISearchBar *)searchBar
{
    [searchBar resignFirstResponder];
}

- (void)searchBarCancelButtonClicked:(UISearchBar *)searchBar
{
    searchBar.showsCancelButton = NO;
    [searchBar resignFirstResponder];
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar
{
    [searchBar resignFirstResponder];

    ContentViewController *browseView = [[ContentViewController alloc] init];
    [self presentViewController:browseView animated:TRUE completion:^
    {
        [browseView search:self.services searchPhrase:self.searchBar.text completionHandler:^
        {
        }];
    }];
}

@end
