//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#import <UIKit/UIKit.h>
#import "CategoryCell.h"
#import "CategoryContentCell.h"
#import "ContentDetailsViewController.h"
#import "ContentViewController.h"
#import "Resources.h"

static CGFloat const CategoryImageWidth = 149;
static CGFloat const CategoryImageHeight = 84;

static NSString *const CollectionViewCellIdentifier = @"ContentCell";

@interface CategoryCell()<UICollectionViewDataSource, UICollectionViewDelegate>

@property (nonatomic, strong) UILabel *nameLabel;
@property (nonatomic, strong) UICollectionView *collectionView;
@property (nonatomic, strong) UIActivityIndicatorView *activityIndicator;
@property (nonatomic, strong) FetchCategoryServicesResult *categoryData;

@end

@implementation CategoryCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier;
{
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier])
    {
        [self setupUI];
        [self.activityIndicator startAnimating];
    }

    return self;
}

- (void)setupUI
{
    self.nameLabel = [[UILabel alloc] init];
    self.nameLabel.textColor = [UIColor whiteColor];
    [self.contentView addSubview:self.nameLabel];
    self.nameLabel.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.nameLabel.heightAnchor constraintEqualToConstant:28],
        [self.nameLabel.leftAnchor constraintEqualToAnchor:self.contentView.leftAnchor constant:0],
        [self.nameLabel.rightAnchor constraintEqualToAnchor:self.contentView.rightAnchor constant:0]]];

    UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
    layout.itemSize = CGSizeMake(CategoryImageWidth, CategoryImageHeight + 20);
    layout.scrollDirection = UICollectionViewScrollDirectionHorizontal;
    self.collectionView = [[UICollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:layout];
    [self.collectionView registerClass:[CategoryContentCell class] forCellWithReuseIdentifier:CollectionViewCellIdentifier];
    self.collectionView.backgroundColor = [UIColor clearColor];
    self.collectionView.showsHorizontalScrollIndicator = YES;
    [self.contentView addSubview:self.collectionView];
    self.collectionView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.collectionView.heightAnchor constraintEqualToConstant:CategoryImageHeight + 20],
        [self.collectionView.topAnchor constraintEqualToAnchor:self.nameLabel.bottomAnchor constant:0],
        [self.collectionView.leftAnchor constraintEqualToAnchor:self.contentView.leftAnchor constant:0],
        [self.collectionView.rightAnchor constraintEqualToAnchor:self.contentView.rightAnchor constant:0]]];

    self.activityIndicator = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleMedium];
    self.activityIndicator.hidesWhenStopped = true;
    self.activityIndicator.color = [Resources mainColor];
    [self.contentView addSubview:self.activityIndicator];
    self.activityIndicator.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.activityIndicator.centerXAnchor constraintEqualToAnchor:self.contentView.centerXAnchor constant:0],
        [self.activityIndicator.centerYAnchor constraintEqualToAnchor:self.contentView.centerYAnchor constant:self.nameLabel.frame.size.height],
    ]];

    self.backgroundColor = [UIColor clearColor];
    self.selectionStyle = UITableViewCellSelectionStyleNone;
}

- (void)updateWithCategoryData:(FetchCategoryServicesResult *)categoryData
{
    self.nameLabel.text = categoryData.category.name;
    self.categoryData = categoryData;

    self.collectionView.dataSource = self;
    self.collectionView.delegate = self;
    [self.collectionView reloadData];
}

- (void)reloadData
{
    [self.collectionView reloadData];
    [self.activityIndicator stopAnimating];
}

- (void)stopLoading
{
    [self.activityIndicator stopAnimating];
}

#pragma mark - UICollectionViewDataSource Methods

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section
{
    NSInteger count = 0;
    for (FetchServicesResult *service in self.categoryData.services)
        count += service.content.count;

    return count;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath
{
    CategoryContentCell *cell = (CategoryContentCell *)[collectionView dequeueReusableCellWithReuseIdentifier:CollectionViewCellIdentifier forIndexPath:indexPath];
    YCSStreamingContent *selectedItem = [self getContentByIndex:indexPath.item];
    YCSStreamingService *service = [self getServiceByIndex:indexPath.item];
    [cell updateWithContentAndService:selectedItem service:service];

    return cell;
}

- (YCSStreamingContent *)getContentByIndex:(NSInteger)index
{
    NSInteger currentIndex = 0;
    for (FetchServicesResult *serviceData in self.categoryData.services)
    {
        for (YCSStreamingContent *content in serviceData.content)
        {
            if (currentIndex == index)
                return content;

            currentIndex++;
        }
    }

    return nil;
}

- (YCSStreamingService *)getServiceByIndex:(NSInteger)index
{
    NSInteger currentIndex = 0;
    for (FetchServicesResult *serviceData in self.categoryData.services)
    {
        currentIndex += serviceData.content.count;
        if (index < currentIndex)
            return serviceData.service;
    }

    return nil;
}

#pragma mark - UICollectionViewDelegate

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath
{
    id <CategoryCellActionDelegate> actionDelegate = self.actionDelegate;
    if (!actionDelegate)
        return;

    YCSStreamingService *service = [self getServiceByIndex:indexPath.item];
    YCSStreamingContent *selectedItem = [self getContentByIndex:indexPath.item];
    if (selectedItem.type == YCSStreamingContentTypeVideo)
        [actionDelegate categoryCellActionDetails:self service:service content:selectedItem];
    else
        [actionDelegate categoryCellActionBrowse:self service:service content:selectedItem];
}

@end
