//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#import "ContentViewController.h"
#import "ContentCell.h"
#import "ContentDetailsViewController.h"
#import "FetchServicesResult.h"
#import "Resources.h"

static NSString *const CellIdentifier = @"BrowseContentCell";

@interface ContentViewController () <UITableViewDataSource, UITableViewDelegate>

@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) UIActivityIndicatorView *activityIndicator;
@property (nonatomic, strong) NSMutableArray<FetchServicesResult *> *servicesData;
@property (nonatomic, strong) YCSConnect *ycsConnect;

@end

@implementation ContentViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.servicesData = [[NSMutableArray<FetchServicesResult *> alloc] init];
    self.ycsConnect = [YCSConnect shared];
    [self setupUI];
}

- (void)browse:(YCSStreamingService *)service content:(YCSStreamingContent *)content completionHandler:(void(^)(void))completionHandler
{
    if (content.type == YCSStreamingContentTypeVideo)
    {
        [self showContentDetails:content];
        if (completionHandler)
            completionHandler();

        return;
    }

    [self.servicesData removeAllObjects];
    FetchServicesResult *serviceData = [[FetchServicesResult alloc] init];
    serviceData.service = service;
    serviceData.content = [[NSMutableArray<YCSStreamingContent *> alloc] init];
    [self.servicesData addObject:serviceData];

    [self.tableView reloadData];
    [self.activityIndicator startAnimating];

    [self.ycsConnect fetchContentChildren:content cancellationTicket:nil completionHandler:^(YCSStreamingContent *content, NSArray<YCSStreamingContent *> *children, NSError *error)
    {
        [serviceData.content addObjectsFromArray:children];

        [self.activityIndicator stopAnimating];
        [self.tableView reloadData];
        if (completionHandler)
            completionHandler();
    }];
}

- (void)search:(NSArray<YCSStreamingService *> *)services searchPhrase:(NSString *)searchPhrase completionHandler:(void(^)(void))completionHandler;
{
    [self.activityIndicator startAnimating];
    [self.servicesData removeAllObjects];
    [self.tableView reloadData];

    [self.ycsConnect searchContent:services
                      searchPhrase:searchPhrase
                cancellationTicket:nil
          serviceCompletionHandler:^(YCSStreamingService *service, NSArray<YCSStreamingContent *> *results, NSError *error)
                                    {
                                        FetchServicesResult *serviceData = [[FetchServicesResult alloc] init];
                                        serviceData.service = service;
                                        serviceData.content = [results mutableCopy];
                                        [self.servicesData addObject:serviceData];
                                        [self.tableView reloadData];
                                    }
              allCompletionHandler:^(NSArray<YCSSearchContentServiceResult *> * _Nonnull results, NSError * _Nullable error)
                                    {
                                        [self.activityIndicator stopAnimating];
                                        if (completionHandler)
                                            completionHandler();
                                    }
    ];
}

- (void)setupUI
{
    self.tableView = [[UITableView alloc] initWithFrame:CGRectZero style:UITableViewStylePlain];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.backgroundColor = [UIColor clearColor];
    self.tableView.rowHeight = 52;
    [self.tableView registerClass:ContentCell.self forCellReuseIdentifier:CellIdentifier];
    [self.view addSubview:self.tableView];
    self.tableView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.tableView.topAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.topAnchor constant:4],
        [self.tableView.leftAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.leftAnchor constant:0],
        [self.tableView.rightAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.rightAnchor constant:0],
        [self.tableView.bottomAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.bottomAnchor constant:0]]];

    self.activityIndicator = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleMedium];
    self.activityIndicator.hidesWhenStopped = TRUE;
    self.activityIndicator.color = [Resources mainColor];
    self.activityIndicator.center = self.view.center;
    [self.view addSubview:self.activityIndicator];
    self.activityIndicator.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.activityIndicator.centerXAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.centerXAnchor constant:0],
        [self.activityIndicator.centerYAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.centerYAnchor constant:0],
    ]];

    self.view.backgroundColor = [UIColor blackColor];
}

- (void)showContentDetails:(YCSStreamingContent *)item
{
    ContentDetailsViewController *detailsView = [[ContentDetailsViewController alloc] initWithContent:item];
    [self presentViewController:detailsView animated:YES completion:nil];
}

#pragma mark - Table view data source

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    NSInteger count = 0;
    for (FetchServicesResult *serviceData in self.servicesData)
        count += serviceData.content.count;

    return count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    ContentCell *cell = (ContentCell *)[tableView dequeueReusableCellWithIdentifier:CellIdentifier];

    if (cell == nil)
        cell = [[ContentCell alloc]initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];

    YCSStreamingContent *item = [self getContentByIndex:indexPath.item];
    YCSStreamingService *service = [self getServiceByIndex:indexPath.item];
    [cell updateWithContentAndService:item service:service];

    return cell;
}

- (YCSStreamingContent *)getContentByIndex:(NSInteger)index
{
    NSInteger currentIndex = 0;
    for (FetchServicesResult *serviceData in self.servicesData)
    {
        for (YCSStreamingContent *content in serviceData.content)
        {
            if (currentIndex == index)
                return content;

            currentIndex++;
        }
    }

    return nil;
}

- (YCSStreamingService *)getServiceByIndex:(NSInteger)index
{
    NSInteger currentIndex = 0;
    for (FetchServicesResult *serviceData in self.servicesData)
    {
        currentIndex += serviceData.content.count;
        if (index < currentIndex)
            return serviceData.service;
    }

    return nil;
}

- (void)tableView:(UITableView *)theTableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    YCSStreamingContent *selectedItem = [self getContentByIndex:indexPath.item];
    YCSStreamingService *service = [self getServiceByIndex:indexPath.item];
    [self browse:service content:selectedItem completionHandler:^
    {
    }];
}

@end
