//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#import "ServicesViewController.h"
#import "ServiceCell.h"
#import "CategoriesViewController.h"
#import "Resources.h"

static NSString *const CellIdentifier = @"ServicesCell";

@interface ServicesViewController () <UITableViewDataSource, UITableViewDelegate, ServiceCellActionDelegate>

@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) UIActivityIndicatorView *activityIndicator;
@property (nonatomic, strong) UIButton *browseButton;

@property (nonatomic, strong) NSArray<YCSStreamingService *> *services;
@property (nonatomic, strong) NSArray<YCSStreamingService *> *linkedServices;
@property (nonatomic, strong) NSArray<YCSStreamingService *> *brokenServices;
@property (nonatomic, strong) YCSStreamingService *selectedService;
@property (nonatomic, strong) YCSConnect *ycsConnect;

@end

@implementation ServicesViewController

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.ycsConnect = [YCSConnect shared];
    [self setupUI];
}

- (void)presentationControllerDidDismiss:(UIPresentationController *)presentationController
{
    self.view.alpha = 1;
    [self refresh];
}

- (void)refresh
{
    [self.activityIndicator startAnimating];
    [self.ycsConnect fetchServices:nil completionHandler:^(NSArray<YCSStreamingService *> *services, NSError *error)
    {
        self.services = services;

        NSPredicate *linkedPredicate = [NSPredicate predicateWithFormat:@"link != nil"];
        self.linkedServices = [self.services filteredArrayUsingPredicate:linkedPredicate];

        NSPredicate *brokenPredicate = [NSPredicate predicateWithFormat:@"link.isBroken == TRUE"];
        NSPredicate *linkBrokenPredicate = [NSCompoundPredicate andPredicateWithSubpredicates:@[linkedPredicate, brokenPredicate]];
        self.brokenServices = [self.services filteredArrayUsingPredicate:linkBrokenPredicate];

        [self.tableView reloadData];
        [self.activityIndicator stopAnimating];
        [self updateBrowseButton];

        if (error != nil)
            [self showError:error];
    }];
}

- (void)showError:(NSError *)error
{
    NSString *description = [error localizedDescription];
    if (error.domain && [error.domain isEqualToString:YCSErrorDomain] && (error.code == YCSErrorNetworkRequestFailed) && [[error.userInfo objectForKey:YCSErrorUserInfoStatusCodeKey] isEqualToNumber:@(401)])
        description = @"Your sdk key or user tokens have not been set or have expired. See AppDelegate.m and TokenManager.m for more details.";

    UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Error" message:description preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *okButton = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil];
    [alert addAction:okButton];
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)setupUI
{
    self.tableView = [[UITableView alloc] initWithFrame:CGRectZero style:UITableViewStylePlain];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.backgroundColor = [UIColor clearColor];
    [self.tableView registerClass:ServiceCell.self forCellReuseIdentifier:CellIdentifier];
    [self.view addSubview:self.tableView];
    self.tableView.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.tableView.topAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.topAnchor constant:4],
        [self.tableView.leftAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.leftAnchor constant:0],
        [self.tableView.rightAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.rightAnchor constant:0],
        [self.tableView.bottomAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.bottomAnchor constant:-30]]];

    self.activityIndicator = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleMedium];
    self.activityIndicator.hidesWhenStopped = TRUE;
    self.activityIndicator.color = [Resources mainColor];
    self.activityIndicator.center = self.view.center;
    [self.view addSubview:self.activityIndicator];
    [self.activityIndicator startAnimating];
    self.activityIndicator.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.activityIndicator.centerXAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.centerXAnchor constant:0],
        [self.activityIndicator.centerYAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.centerYAnchor constant:0],
    ]];

    self.browseButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.browseButton setTitleColor:[UIColor colorWithRed:32.0/255.0 green:32.0/255.0 blue:32.0/255.0 alpha:1.0] forState:UIControlStateNormal];
    self.browseButton.backgroundColor = [Resources mainColor];
    self.browseButton.layer.borderColor = [UIColor clearColor].CGColor;
    self.browseButton.layer.borderWidth = 0.0f;
    self.browseButton.layer.cornerRadius = 10.0f;
    self.browseButton.titleLabel.font = [UIFont systemFontOfSize:12];
    [self.browseButton addTarget:self action:@selector(browseClicked:) forControlEvents:UIControlEventTouchUpInside];
    [self.browseButton setTitle:@"Browse All" forState:UIControlStateNormal];
    self.browseButton.hidden = TRUE;
    [self.view addSubview:self.browseButton];
    self.browseButton.translatesAutoresizingMaskIntoConstraints = NO;
    [NSLayoutConstraint activateConstraints:@[
        [self.browseButton.widthAnchor constraintEqualToConstant:80],
        [self.browseButton.heightAnchor constraintEqualToConstant:26],
        [self.browseButton.topAnchor constraintEqualToAnchor:self.tableView.bottomAnchor constant:0],
        [self.browseButton.centerXAnchor constraintEqualToAnchor:self.view.safeAreaLayoutGuide.centerXAnchor constant:0]]];

    self.view.backgroundColor = [UIColor blackColor];
}

- (void)updateBrowseButton
{
    BOOL isBrowseable = self.linkedServices && (self.linkedServices.count > 0);
    self.browseButton.hidden = !isBrowseable;
}

- (void)browseClicked:(UIButton *)sender
{
    CategoriesViewController *younifyView = [[CategoriesViewController alloc] initWithServices:self.linkedServices];
    younifyView.presentationController.delegate = self;
    self.view.alpha = 0.2;
    [self presentViewController:younifyView animated:TRUE completion:nil];
}

#pragma mark - ServiceCellActionDelegate

- (void)serviceCellActionBrowse:(ServiceCell *)sender
{
    NSArray<YCSStreamingService *> *services = [[NSArray<YCSStreamingService *> alloc] initWithObjects:sender.service, nil];
    CategoriesViewController *younifyView = [[CategoriesViewController alloc] initWithServices:services];
    younifyView.presentationController.delegate = self;
    self.view.alpha = 0.2;
    [self presentViewController:younifyView animated:TRUE completion:nil];
}

- (void)serviceCellActionLogout:(ServiceCell *)sender
{
    [self.activityIndicator startAnimating];

    [self.ycsConnect unlinkService:sender.service cancellationTicket:nil completionHandler:^(YCSStreamingService *service, NSError *error)
    {
        [self.activityIndicator stopAnimating];

        [sender updateWithService:service];
    }];
}

#pragma mark - Table view data source

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.services.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    ServiceCell *cell = (ServiceCell *)[tableView dequeueReusableCellWithIdentifier:CellIdentifier];

    if (cell == nil)
        cell = [[ServiceCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];

    YCSStreamingService *service = self.services[indexPath.row];
    [cell updateWithService:service];
    cell.actionDelegate = self;

    return cell;
}

- (void)tableView:(UITableView *)theTableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    YCSStreamingService *service = self.services[indexPath.row];
    if (!service.isAvailable)
    {
        [self showError:[NSError errorWithDomain:@"svc" code:0 userInfo:@{ NSLocalizedDescriptionKey: [NSString stringWithFormat:@"Sorry, %@ is currently unavailable.", service.name] }]];
        [self.tableView deselectRowAtIndexPath:indexPath animated:FALSE];
        return;
    }

    [self.activityIndicator startAnimating];

    BOOL isLinked = service.link && !service.link.isBroken;
    if (isLinked)
    {
        [self.ycsConnect manageLinkedService:service cancellationTicket:nil completionHandler:^(YCSStreamingService *service, NSError *error)
        {
            [self.activityIndicator stopAnimating];

            if (error != nil)
                [self showError:error];

            [self refresh];
        }];
    }
    else
    {
        [self.ycsConnect linkService:service cancellationTicket:nil completionHandler:^(YCSStreamingService *service, BOOL success, NSError *error)
        {
            if (error != nil)
                [self showError:error];

            [self.activityIndicator stopAnimating];
            [self refresh];
        }];
    }
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 50;
}

@end
